package processModel;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.geom.Line2D;
import java.awt.geom.QuadCurve2D;

import processView.Externaliser;
import utilities.AppConstants;

public class AdiabaticTransition extends Transition {
	private QuadCurve2D line, crescentBoundary1, crescentBoundary2;
	private Polygon arrowHead;
	private final float ARROW_STROKE = 2;
	
	public AdiabaticTransition(String type){
		super(type);
	}

	@Override
	protected void createTransitionShape() {
		try{
		//		 line
		
		line = new QuadCurve2D.Double();
		Point controlPoint = getControlPoint();
		line.setCurve(beginPoint(), controlPoint, endPoint());
		
		// Determine crescent boundaries
		double x1 = beginPoint().getX();
		double y1 = beginPoint().getY();
		double x2 = endPoint().getX();
		double y2 = endPoint().getY();
		double dirX = x1-x2;
		double dirY = y1-y2;
		double perpDirX = dirY;
		double perpDirY = -dirX;
		double len = Math.sqrt(perpDirX*perpDirX + perpDirY*perpDirY);
		perpDirX = perpDirX / len * 15;
		perpDirY = perpDirY / len * 15;
		
		crescentBoundary1 = new QuadCurve2D.Double();
		Point controlPointOuter = new Point((int)(getControlPoint().x+perpDirX), (int)(getControlPoint().y+perpDirY));
		crescentBoundary1.setCurve(beginPoint(), controlPointOuter, endPoint());
		
		crescentBoundary2 = new QuadCurve2D.Double();
		Point controlPointInner = new Point((int)(getControlPoint().x-perpDirX), (int)(getControlPoint().y-perpDirY));
		crescentBoundary2.setCurve(beginPoint(), controlPointInner, endPoint());
		
			// arrow head
	      arrowHead = new Polygon();
	      double aDir=Math.atan2(controlPoint.x-endPoint().x,controlPoint.y-endPoint().y);
	      int i1=12+(int)(ARROW_STROKE*2);
	      int i2=6+(int)ARROW_STROKE;							// make the arrow head the same size regardless of the length length
	      arrowHead.addPoint(endPoint().x,endPoint().y);							// arrow tip
	      arrowHead.addPoint(endPoint().x+xCor(i1,aDir+.5),endPoint().y+yCor(i1,aDir+.5));
	      arrowHead.addPoint(endPoint().x+xCor(i2,aDir),endPoint().y+yCor(i2,aDir));
	      arrowHead.addPoint(endPoint().x+xCor(i1,aDir-.5),endPoint().y+yCor(i1,aDir-.5));
	      arrowHead.addPoint(endPoint().x,endPoint().y);
		} catch (Exception e){}
	}
	
	private static int yCor(int len, double dir) {return (int)(len * Math.cos(dir));}
	private static int xCor(int len, double dir) {return (int)(len * Math.sin(dir));}

	private Point getControlPoint(){		
		Point cPoint = new Point();		
		double dx = (double)endPoint().x - (double)beginPoint().x;
		double dy = (double)endPoint().y - (double)beginPoint().y;
		double L = Math.sqrt(dx*dx + dy*dy);
		double curvex, curvey;
		double m = dy/dx;
		double m_pd = -1/m;
		double midx = beginPoint().x + (endPoint().x - beginPoint().x)/2;
		double midy = beginPoint().y + (endPoint().y - beginPoint().y)/2;
		
		if(beginPoint().x == endPoint().x || beginPoint().y == endPoint().y){
			curvex = midx; curvey=midy;}
		else if(beginPoint().x <endPoint().x)
		{
			if(Math.abs(m)<=Math.abs(m_pd)) 
				if(endPoint().y>beginPoint().y)
				{
					curvex = midx - Math.abs(m)*L/4;
					curvey =  m_pd * (curvex - midx) + midy;
				}
				else 
				{
					curvex = midx + Math.abs(m)*L/4;
					curvey =  m_pd * (curvex - midx) + midy;
				}
			
			else 
				if(endPoint().y>beginPoint().y)
				{
					curvey = midy + Math.abs(m_pd)*L/4;
					curvex = (curvey - midy)/m_pd + midx;
				}
				else 
				{
					curvey = midy + Math.abs(m_pd)*L/4;
					curvex = (curvey - midy)/m_pd + midx;
				}
		}
		else{
			if(Math.abs(m)<=Math.abs(m_pd)) 
				if(endPoint().y>beginPoint().y)
				{
					curvex = midx + Math.abs(m)*L/4;
					curvey =  m_pd * (curvex - midx) + midy;
				}
				else {
					curvex = midx - Math.abs(m)*L/4;
					curvey =  m_pd * (curvex - midx) + midy;
				}
			else 
				if(endPoint().y>beginPoint().y)
				{
					curvey = midy + Math.abs(m_pd)*L/4;
					curvex = (curvey - midy)/m_pd + midx;
				}
				else 
				{
					curvey = midy + Math.abs(m_pd)*L/4;
					curvex = (curvey - midy)/m_pd + midx;
				}
		}
		cPoint.setLocation(curvex, curvey);
		return cPoint;
	}
	@Override
	public void move(Point p) {
	}

	@Override
	public void paint(Graphics2D g2) {
		Graphics2D g2d = (Graphics2D)g2.create(); //Make a copy so original is unchanged
		
		if (line != null) {
			g2d.setColor(AppConstants.ADIABATIC_COLOR);
			g2d.draw(line);
			g2d.draw(arrowHead);
			g2d.fill(arrowHead);
			g2d.setColor(Color.BLACK);
			Font labelFont = new Font("Arial",Font.BOLD,10);
			g2d.setFont(labelFont);
			if(this.label()!=null){
				double moveAmount = getLineLength() / 10;
				double moveAmountX = -moveAmount * 0.5;
				double moveAmountY = moveAmount * 0.6;
				double midx = beginPoint().x + (endPoint().x - beginPoint().x)/2 + moveAmountX;
				double midy = beginPoint().y + (endPoint().y - beginPoint().y)/2 + moveAmountY;
				g2d.drawString(this.label(), (int)midx, (int)midy);
			}
			for (Handle handle : getHandles()){
				handle.paint(g2d);
			}
		}
		g2d.dispose();
	}

	@Override
	public Element pointIsInElement (Point p) {
		if (line != null) {
			// Check to see if it's within the cresent
			boolean withinCresent;
			if (line.getX1() > line.getX2()) {
				withinCresent = crescentBoundary2.contains(p) && !crescentBoundary1.contains(p);
			} else {
				withinCresent = crescentBoundary1.contains(p) && !crescentBoundary2.contains(p);
			}
			if(withinCresent ||
					(line.getP1().getX() == p.getX() && line.getP1().getY() == p.getY()) 
					|| (line.getP2().getX() == p.getX() && line.getP2().getX() == p.getX()))
				return this;
			else
				return null;
		}
		return null;
	}
	
	public String toXML(String indent) {
		String xmlStr = "";
		if(beginPoint().x <= endPoint().x){
			xmlStr += Externaliser.openTag(indent, AppConstants.ELT_ADIABATIC_E + AppConstants.S + AppConstants.ATT_ID
					+ Externaliser.quoted(String.valueOf(this.getID())));
			xmlStr+=transitionXMLContent(indent);		
			xmlStr += Externaliser.closeTag(indent, AppConstants.ELT_ADIABATIC_E);
		}else{
			xmlStr += Externaliser.openTag(indent, AppConstants.ELT_ADIABATIC_C + AppConstants.S + AppConstants.ATT_ID
					+ Externaliser.quoted(String.valueOf(this.getID())));
			xmlStr+=transitionXMLContent(indent);		
			xmlStr += Externaliser.closeTag(indent, AppConstants.ELT_ADIABATIC_C);
		}
		return xmlStr;
	}
	
	public String toASPIRE_XML(String indent) {
		String xmlStr = "";
		if(beginPoint().x <= endPoint().x){
			xmlStr +=  Externaliser.openInstanceTag(indent, this.getID(), "Adiabatic expansion");
			xmlStr += TransitionASPIRE_XML(indent);
			xmlStr +=  Externaliser.closeInstanceTag(indent);
		}else{
			xmlStr +=  Externaliser.openInstanceTag(indent, this.getID(), "Adiabatic compression");
			xmlStr += TransitionASPIRE_XML(indent);
			xmlStr +=  Externaliser.closeInstanceTag(indent);

		}
		
		return xmlStr;
	}		
}
