package processModel;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.geom.Line2D;

import processView.Externaliser;
import utilities.AppConstants;

public class IsobaricTransition extends Transition {


	private Line2D.Double line;
	private Polygon arrowHead;
	private final float ARROW_STROKE = 2;
	
	public IsobaricTransition(String type){
		super(type);
	}

	@Override
	protected void createTransitionShape() {
		try{
		// line
	      line = new Line2D.Double(beginPoint().x,beginPoint().y, endPoint().x, endPoint().y);
		// arrow head
	      arrowHead = new Polygon();
	      double aDir=Math.atan2(beginPoint().x-endPoint().x,beginPoint().y-endPoint().y);
	      int i1=12+(int)(ARROW_STROKE*2);
	      int i2=6+(int)ARROW_STROKE;							// make the arrow head the same size regardless of the length length
	      arrowHead.addPoint(endPoint().x,endPoint().y);							// arrow tip
	      arrowHead.addPoint(endPoint().x+xCor(i1,aDir+.5),endPoint().y+yCor(i1,aDir+.5));
	      arrowHead.addPoint(endPoint().x+xCor(i2,aDir),endPoint().y+yCor(i2,aDir));
	      arrowHead.addPoint(endPoint().x+xCor(i1,aDir-.5),endPoint().y+yCor(i1,aDir-.5));
	      arrowHead.addPoint(endPoint().x,endPoint().y);
		}catch (Exception e){}
	}


	@Override
	public void move(Point p) {		
	}

	@Override
	public void paint(Graphics2D g2) {
		// TODO: Reliance on this test is fishy. Instead, Element interface
		// should become a class and implement isVisible and setVisible methods.
		// Then only visible Elements can be drawn.
		Graphics2D g2d = (Graphics2D)g2.create(); //Make a copy so original is unchanged
	
		if (line != null) {
			g2d.setColor(AppConstants.ISOBARIC_COLOR);
			g2d.draw(line);
			g2d.draw(arrowHead);
			g2d.fill(arrowHead);
			g2d.setColor(Color.BLACK);
			Font labelFont = new Font("Arial",Font.BOLD,10);
			g2d.setFont(labelFont);
			if(this.label()!=null){
				double midx = beginPoint().x + (endPoint().x - beginPoint().x)/2 - 10;
				double midy = beginPoint().y + (endPoint().y - beginPoint().y)/2 - 5;
				g2d.drawString(this.label(), (int)midx, (int)midy);
			}
			for (Handle handle : getHandles()){
				handle.paint(g2d);
			}
		}
		g2d.dispose();
		
	}

	@Override
	public Element pointIsInElement(Point p) {
		if (line != null)
			if (line.ptLineDist(p) < 6)
				return this;
			else
				return null;
		return null;
	}

	private static int yCor(int len, double dir) {return (int)(len * Math.cos(dir));}
	private static int xCor(int len, double dir) {return (int)(len * Math.sin(dir));}
	
	public String toString(){
		String lineDescr = "";
		try{
			lineDescr = "line: [[" + line.x1 + "," + line.y1 + "],[" + line.x2 + "," + line.y2 + 
			"]]";
		}catch (Exception e){}
		 return super.toString() + lineDescr;
	}
	
	public String toXML(String indent) {
		String xmlStr = "";
		if(beginPoint().x <= endPoint().x){
			xmlStr += Externaliser.openTag(indent, AppConstants.ELT_ISOBARTIC_E + AppConstants.S + AppConstants.ATT_ID
					+ Externaliser.quoted(String.valueOf(this.getID())));
			xmlStr+=transitionXMLContent(indent);
			xmlStr += Externaliser.closeTag(indent, AppConstants.ELT_ISOBARTIC_E);
			}else{
				xmlStr += Externaliser.openTag(indent, AppConstants.ELT_ISOBARTIC_C + AppConstants.S + AppConstants.ATT_ID
						+ Externaliser.quoted(String.valueOf(this.getID())));
				xmlStr+=transitionXMLContent(indent);
				xmlStr += Externaliser.closeTag(indent, AppConstants.ELT_ISOBARTIC_C);
			}
		return xmlStr;
	}
	
	public String toASPIRE_XML(String indent) {
		String xmlStr = "";
		if(beginPoint().x <= endPoint().x){
			xmlStr +=  Externaliser.openInstanceTag(indent, this.getID(), "Isobaric expansion");
			xmlStr += TransitionASPIRE_XML(indent);
			xmlStr +=  Externaliser.closeInstanceTag(indent);
		}else{
			xmlStr +=  Externaliser.openInstanceTag(indent, this.getID(), "Isobaric compression");
			xmlStr += TransitionASPIRE_XML(indent);
			xmlStr +=  Externaliser.closeInstanceTag(indent);
		}
		
		return xmlStr;
	}
}

