package processModel;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Observable;

import processModel.StateUnknown.StateUnknown_Type;
import processModel.TransitionUnknown.TransitionUnknown_Type;
import processView.Externaliser;
import utilities.AppConstants;

public class StateElement implements Element, Moveable {
	
	
	private List<StateHandle> handles;
	private String label = "";
	private Ellipse2D circle, mask;
	private boolean selected;
	//private Point firstPoint, secondPoint;
	private Rectangle circleBounds;
	private final int MASK_INCREMENT = 5;
	private String temperature = "";
	private String volume = "";
	private String pressure = "";
	private String temperatureUnits = "";
	private String volumeUnits = "";
	private String pressureUnits = "";
	private boolean temperatureCalculated = false;
	private boolean volumeCalculated = false;
	private boolean pressureCalculated = false;
	private List<Unknown> calculatedValues = null;;
	protected int id = -1;
	
	public StateElement(){}
	
	/*public StateElement(Point p1, Point p2){
		createCircleBounds(p1,p2);
		createStateShape();
		createHandles();
	}*/
	
	private void createCircleBounds(Point p1, Point p2){
		circleBounds = new Rectangle (p1);
		circleBounds.add(p2);
		circleBounds.width = AppConstants.MINIMUM_STATE_SIZE;
		circleBounds.height = AppConstants.MINIMUM_STATE_SIZE;
	}

	private void createStateShape (){
			
			// create the circle
			circle = new Ellipse2D.Double();
			circle.setFrame(circleBounds);	
			
			
			//create the mask
			//Point maskFirstPoint = new Point (((int)firstPoint.getX() - MASK_INCREMENT), ((int)firstPoint.getY() - MASK_INCREMENT));
			//Point maskSecondPoint = new Point (((int)secondPoint.getX() + MASK_INCREMENT), ((int)secondPoint.getY() + MASK_INCREMENT));
			Rectangle maskFrame = new Rectangle (circleBounds);
			maskFrame.grow(MASK_INCREMENT, MASK_INCREMENT);
			mask = new Ellipse2D.Double();
			mask.setFrame(maskFrame);
	}
	
	public boolean selected() {
		return selected;
	}

	public void paint(Graphics2D g2) {
		if (circle != null){
			g2.setColor(AppConstants.STATE_FILL_COLOR);
			g2.fill(circle);
			g2.setColor(AppConstants.STATE_BOUNDARY_COLOR);
			g2.draw(circle);
			g2.setColor(Color.BLACK);
			Font labelFont = new Font("Arial",Font.BOLD,10);
			g2.setFont(labelFont);
			g2.drawString(this.label(), (float)circle.getCenterX()-2, (float)circle.getCenterY()+4);
			if (handles != null){
				Iterator <StateHandle> handleIt = handles.iterator();
				while (handleIt.hasNext()){
					handleIt.next().paint(g2);
				}
			}
		}
	}
	
	public Rectangle getCircleBounds()
	{
		Rectangle bounds = new Rectangle(circleBounds);
		bounds.x += bounds.width / 2;
		bounds.y += bounds.height / 2;
		return bounds;
	}

	public void setEndPoint(Point p) {
		Point startPoint = p;
		if (circleBounds != null)
			startPoint = circleBounds.getLocation();
		createCircleBounds(startPoint,p);
		this.createStateShape();
		createHandles();
	}
	
	public void setOrigin(Point p) {
		Point endPoint = p;
		if (circleBounds != null)
			endPoint = new Point( circleBounds.x + circleBounds.width, circleBounds.y + circleBounds.height);
		this.createCircleBounds(p, endPoint);
		this.createStateShape();
		createHandles();
	}
	
	public void setCenter(Point p)
	{
		Point startPoint = new Point( p.x - (AppConstants.MINIMUM_STATE_SIZE/2), p.y - (AppConstants.MINIMUM_STATE_SIZE/2));
		Point endPoint = new Point( p.x + (AppConstants.MINIMUM_STATE_SIZE/2), p.y + (AppConstants.MINIMUM_STATE_SIZE/2));
		this.createCircleBounds(startPoint, endPoint);
		
		this.createStateShape();
		createHandles();
	}

	public void label(String l) {
		this.label = l;
	}
	
	public void setHandles(List<StateHandle> handles) {
		this.handles = handles;
	}
	
	public String label(){
		return label;
	}

	// Is it good to have accessors? Have to decide later.
	public String temperature()
	{
		if (temperatureUnknown()) {
			return "";
		} else {
			return this.temperature;	
		}
	}
	public String pressure()
	{
		if (pressureUnknown()) {
			return "";
		} else {
			return this.pressure;	
		}
	}
	public String volume()
	{
		if (volumeUnknown()) {
			return "";
		} else {
			return this.volume;	
		}
	}
	
	public List<Unknown> getUnknowns(){
		if (calculatedValues == null) {
			calculatedValues = new ArrayList<Unknown>();
			if (pressureUnknown())
				calculatedValues.add(new StateUnknown(this, Process.getNextID(), StateUnknown_Type.Pressure));
			if (volumeUnknown())
				calculatedValues.add(new StateUnknown(this, Process.getNextID(), StateUnknown_Type.Volume));
			if (temperatureUnknown())
				calculatedValues.add(new StateUnknown(this, Process.getNextID(), StateUnknown_Type.Temperature));
		}
		List<Unknown> unknowns = new ArrayList<Unknown>(calculatedValues);
		ListIterator<Unknown> iter = unknowns.listIterator();
		while (iter.hasNext())
		{
			if (!iter.next().getUnknownValue().equals("")) {
				iter.remove();
			}
		}
		return unknowns;
    }
	
	public List<Unknown> getCalculatedValues()
	{
		List<Unknown> result = new ArrayList<Unknown>(calculatedValues);
		ListIterator<Unknown> iter = result.listIterator();
		while (iter.hasNext())
		{
			Unknown next = iter.next();
			if (next.getUnknownValue().equals("")) {
				iter.remove();
				System.out.println("Unknown Removed: " + next.getID());
			}
			else {
				System.out.println("Unknown Kept: " + next.getID());
			}
		}
		return result;
	}
	
	public String getTemperatureUnits() {
		return temperatureUnits;
	}
	
	public String getPressureUnits() {
		return pressureUnits;
	}
	
	public String getVolumeUnits() {
		return volumeUnits;
	}
	
	public void temperature (String temp){this.temperature = temp.trim();}
	public void pressure (String pressure){this.pressure = pressure.trim();}
	public void volume (String volume){this.volume = volume.trim();}
	public void temperatureUnit(String tempUnit){this.temperatureUnits = tempUnit.trim();}
	public void pressureUnit(String pressureUnit){this.pressureUnits = pressureUnit.trim();}
	public void volumeUnit(String volumeUnit){this.volumeUnits = volumeUnit.trim();}
	
	public void setCalculatedTemperature(String temp)
	{
		temperature(temp);
		temperatureCalculated = true;
	}
	
	public void setCalculatedPressure(String pressure)
	{
		pressure(pressure);
		pressureCalculated = true;
	}
	
	public void setCalculatedVolume(String volume)
	{
		volume(volume);
		volumeCalculated = true;
	}
	
	public boolean temperatureCalculated()
	{
		return temperatureCalculated;
	}
	
	public boolean pressureCalculated()
	{
		return pressureCalculated;
	}
	
	public boolean volumeCalculated()
	{
		return volumeCalculated;
	}
	
	public boolean temperatureUnknown() {
		return this.temperature == null;
	}
	
	public void setTemperatureUnknown() {
		this.temperature = null;
	}	
	
	public boolean pressureUnknown() {
		return this.pressure == null;
	}
	
	public void setPressureUnknown() {
		this.pressure = null;
	}
	
	public boolean volumeUnknown() {
		return this.volume == null;
	}
	
	public void setVolumeUnknown() {
		this.volume = null;
	}
	

	public Point2D getCenter (){
		Point2D c = null;
		if (circle != null)
			c = new Point((int) circle.getCenterX(), (int)circle.getCenterY());
		return c;
	}
	

	public void setBeginPoint(Point p) {
		this.setOrigin(p);
	}

	public List<StateHandle> getHandles() {
		return handles;
	}

	private void createHandles(){
		
		int middleX = circleBounds.x + (circleBounds.width/2);
		int endX = circleBounds.x + circleBounds.width;
		int middleY = circleBounds.y + (circleBounds.height/2);
		int endY = circleBounds.y + circleBounds.height;
		
		// Top central point
		Point p1 = new Point (middleX, circleBounds.y);
		StateHandle handle1 = new StateHandle(p1, this);
		handle1.setCursorType(Cursor.N_RESIZE_CURSOR);
		handle1.addObs(this);
		
		// Right central point
		Point p2 = new Point (endX, middleY);
		StateHandle handle2 = new StateHandle(p2, this);
		handle2.setCursorType(Cursor.E_RESIZE_CURSOR);
		handle2.addObs(this);
		
		// bottom central point
		Point p3 = new Point(middleX, endY);
		StateHandle handle3 = new StateHandle(p3, this);
		handle3.setCursorType(Cursor.S_RESIZE_CURSOR);
		handle3.addObs(this);
		
		// left central point
		Point p4 = new Point (circleBounds.x, middleY);
		StateHandle handle4 = new StateHandle(p4, this);
		handle4.setCursorType(Cursor.W_RESIZE_CURSOR);
		handle4.addObs(this);
		
		handles = new ArrayList<StateHandle>();
		
		handles.add(handle1);
		handles.add(handle2);
		handles.add(handle3);
		handles.add(handle4);
		
	}

	// use this method to move the existing handles.
	// CircleBounds must have been created before calling this method
	private void moveHandles (){
		
		// first check that handles exist. If not, create them instead.
		/*if (handles.size() < 4){
			createHandles();
			return;
		}*/
		
		int middleX = circleBounds.x + (circleBounds.width/2);
		int endX = circleBounds.x + circleBounds.width;
		int middleY = circleBounds.y + (circleBounds.height/2);
		int endY = circleBounds.y + circleBounds.height;
		
		// Top central point
		Point p0 = new Point (middleX, circleBounds.y);
		handles.get(0).move(p0);
		handles.get(0).setCursorType(Cursor.N_RESIZE_CURSOR);
		
		// Right central point
		Point p1 = new Point (endX, middleY);
		handles.get(1).move(p1);
		handles.get(1).setCursorType(Cursor.E_RESIZE_CURSOR);
		
		// bottom central point
		Point p2 = new Point(middleX, endY);
		handles.get(2).move(p2);
		handles.get(2).setCursorType(Cursor.S_RESIZE_CURSOR);
		
		// left central point
		Point p3 = new Point (circleBounds.x, middleY);
		handles.get(3).move(p3);
		handles.get(3).setCursorType(Cursor.W_RESIZE_CURSOR);
		
	}
	public void select(boolean select) {
		this.selected = select;
		if (handles != null)
		for (StateHandle handle:handles)
				handle.select(select);
	}
	
	public Handle getClosestHandle(Point p){
		Handle closestHandle = null;
		Iterator <StateHandle> handleIt = handles.iterator();
		double shortestDistanceBetweenPoints = Integer.MAX_VALUE;
		
		while (handleIt.hasNext()){
			Handle handle = handleIt.next();
			Point handlePoint = handle.origin();
			double distance = p.distance(handlePoint);
			if (distance < shortestDistanceBetweenPoints){
				shortestDistanceBetweenPoints = distance;
				closestHandle = handle;
			}
		}
		return closestHandle;
	}

	public void update(Observable arg0, Object arg1) {
	}
	
	public Element pointIsInElement(Point p){
		if (mask == null) return null;
		if (mask.contains(p))
			return this;
		else return null;
	}

	public Handle pointIsInHandle (Point p){

		Handle selectedHandle = null;
		
		for (Handle tempHandle : handles){
			if (tempHandle.pointIsInHandle(p) != null){
				return selectedHandle = tempHandle.pointIsInHandle(p);
			}
		}
		return selectedHandle;
	}
	
	public void prepareForDeletion() {
		for (StateHandle handle : handles){
			handle.prepareForDeletion();
			handle = null;
		}
		handles = null;
	}
	
	public int getID ()
	{
		return id;
	}
	
	public void setID(int id)
	{
		this.id = id;
	}
	
	/*public boolean deleted ()
	{
		return id == -1;
	}*/

	public void move(Point p) {
		/*System.out.println(">>>MOVE: In move, point is: " + p + " and original location is " + circleBounds.getLocation());
		
		this.circleBounds.setLocation(p.x - (circleBounds.width / 2), p.y - (circleBounds.height / 2));
		this.createStateShape();
		this.moveHandles();*/
		
		Point startPoint = new Point( p.x - (AppConstants.MINIMUM_STATE_SIZE/2), p.y - (AppConstants.MINIMUM_STATE_SIZE/2));
		this.circleBounds.setLocation(startPoint);
		this.createStateShape();
		this.moveHandles();
		
		// First get the bounds of the current circle
		//	Rectangle currentBounds = new Rectangle (circle.getBounds());
		// Then set the new frame of the circle to the new bounds
		//circle.setFrame(p.getY(), p.getY(), currentBounds.getWidth(), currentBounds.getHeight());
	}
	
	public String toXML(String indent) {
		String xmlStr = "";
		
		xmlStr += Externaliser.openTag(indent, AppConstants.ELT_STATE + AppConstants.S + AppConstants.ATT_ID 
				+ Externaliser.quoted(String.valueOf(this.getID())));
		xmlStr += stateXMLContent(indent);		
		xmlStr += Externaliser.closeTag(indent, AppConstants.ELT_STATE);		
		return xmlStr;
	}
	
	protected String stateXMLContent(String indent) {
    	String xmlStr = "";
				
		// Label and magnitudes
		if (!"".equals(this.label)) {
			xmlStr += Externaliser.emptyTag(indent + AppConstants.Sx2, AppConstants.ELT_LABEL + AppConstants.S + AppConstants.ATT_VALUE +
					Externaliser.quoted(this.label()));
		}
		
		xmlStr += Externaliser.emptyTag(indent + AppConstants.Sx2, AppConstants.ELT_TEMPERATURE 
					+ AppConstants.S + AppConstants.ATT_VALUE + Externaliser.quoted(this.temperature()) 
					+ AppConstants.S + AppConstants.ATT_UNKNOWN + Externaliser.quoted(this.temperatureUnknown()));
		
		if (!"".equals(this.temperatureUnits)) {
			xmlStr += Externaliser.emptyTag(indent + AppConstants.Sx2, AppConstants.ELT_UNITS + AppConstants.S + AppConstants.ATT_VALUE +
					Externaliser.quoted(this.getTemperatureUnits()));
		}
				
		xmlStr += Externaliser.emptyTag(indent + AppConstants.Sx2, AppConstants.ELT_PRESSURE 
				+ AppConstants.S + AppConstants.ATT_VALUE + Externaliser.quoted(this.pressure())
				+ AppConstants.S + AppConstants.ATT_UNKNOWN + Externaliser.quoted(this.pressureUnknown()));
		
		if (!"".equals(this.pressureUnits)) {
			xmlStr += Externaliser.emptyTag(indent + AppConstants.Sx2, AppConstants.ELT_UNITS + AppConstants.S + AppConstants.ATT_VALUE +
					Externaliser.quoted(this.getPressureUnits()));
		}		
		
		xmlStr += Externaliser.emptyTag(indent + AppConstants.Sx2, AppConstants.ELT_VOLUME 
				+ AppConstants.S + AppConstants.ATT_VALUE + Externaliser.quoted(this.volume())
				+ AppConstants.S + AppConstants.ATT_UNKNOWN + Externaliser.quoted(this.volumeUnknown()));
		
		if (!"".equals(this.volumeUnits)) {
			xmlStr += Externaliser.emptyTag(indent + AppConstants.Sx2, AppConstants.ELT_UNITS + AppConstants.S + AppConstants.ATT_VALUE +
					Externaliser.quoted(this.getVolumeUnits()));
		}
		
        return xmlStr;
    	
    }
	
	public String toASPIRE_XML(String indent) {
		String xmlStr = "";
		
		xmlStr +=  Externaliser.openInstanceTag(indent, this.getID(), "State");
		xmlStr += stateASPIRE_XML(indent);
		xmlStr +=  Externaliser.closeInstanceTag(indent);							
		
		return xmlStr;
    }

	protected String stateASPIRE_XML(String indent) {
		String xmlStr = "";
		
		// Label
		xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 0, this.label());		

		// Pressure
		xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 4, this.pressure());
		// Units
		xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 6, this.getPressureUnits());
		// Pressure Unknown
		xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 5, String.valueOf(this.pressureUnknown()));
		
		
		// Volume 
		xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 7, this.volume());
		// Units 
		xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 9, this.getVolumeUnits());
		// Volume Unknown
		xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 8, String.valueOf(this.volumeUnknown()));
		
		
		// Temperature 
		xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 1, this.temperature());
		// Units
		xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 3, this.getTemperatureUnits());
		// Temperature Unknown
		xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 2, String.valueOf(this.temperatureUnknown()));
		
		return xmlStr;
	}

	// TODO:  Get rid of these from super class
	@Override
	public void delta(String delta) {
	}

	// TODO:  Get rid of these from super class
	@Override
	public String delta() {
		return null;
	}
}