package processModel;

import java.text.DecimalFormat;

import processModel.StateUnknown.StateUnknown_Type;
import processView.Externaliser;
import utilities.AppConstants;

public class TransitionUnknown implements Unknown
{
	public enum TransitionUnknown_Type
	{
		DeltaU("Delta U", "U-Unknown"), Heat("Heat", "Q-Unknown"), Work("Work", "W-Unknown");
		
		private String name;
		private String aspireName;
		
		private TransitionUnknown_Type(String name, String aspireName)
		{
			this.name = name;
			this.aspireName = aspireName;
		}
		
		public String getName() { return name; }
		
		public String getAspireName() { return aspireName; }
	}
	
	private TransitionUnknown_Type type;
	private int id;
	private Transition transition;
	private boolean beingWorkedOn = false;
	
	public TransitionUnknown(Transition transition, int id, TransitionUnknown_Type type)
	{
		this.type = type;
		this.id = id;
		this.transition = transition;
	}
	
	public void setUnknownValue(String value, String units)
	{
		if (type.equals(TransitionUnknown_Type.DeltaU)) {
			transition.setCalculatedDelta(value);
			transition.deltaUnit(units);
		}
		else if (type.equals(TransitionUnknown_Type.Heat)) {
			transition.setCalculatedHeat(value);
			transition.heatUnit(units);
		}
		else if (type.equals(TransitionUnknown_Type.Work)) {
			transition.setCalculatedWork(value);
			transition.workUnit(units);
		}
	}
	
	public void clearUnknownValue()
	{
		if (type.equals(TransitionUnknown_Type.DeltaU)) {
			transition.setDeltaUnknown();
			transition.deltaUnit("");
		}
		else if (type.equals(TransitionUnknown_Type.Heat)) {
			transition.setHeatUnknown();
			transition.heatUnit("");
		}
		else if (type.equals(TransitionUnknown_Type.Work)) {
			transition.setWorkUnknown();
			transition.workUnit("");
		}
	}
	
	public String getUnknownValue()
	{
		String result = "";
		if (type.equals(TransitionUnknown_Type.DeltaU)) {
			result = transition.delta();
		}
		else if (type.equals(TransitionUnknown_Type.Heat)) {
			result = transition.heat();
		}
		else if (type.equals(TransitionUnknown_Type.Work)) {
			result = transition.work();
		}
		return Process.roundNumber(result);
	}
	
	public String getUnknownUnit()
	{
		String result = "";
		if (type.equals(TransitionUnknown_Type.DeltaU)) {
			result = transition.getDeltaUnits();
		}
		else if (type.equals(TransitionUnknown_Type.Heat)) {
			result = transition.getHeatUnits();
		}
		else if (type.equals(TransitionUnknown_Type.Work)) {
			result = transition.getWorkUnits();
		}
		return result;
	}
	
	public int getID()
	{
		return id;
	}
	
	public String toASPIRE_XML(String indent) {
		String xmlStr = "";
		
		xmlStr += Externaliser.openInstanceTag(indent, this.getID(), type.getAspireName());
		xmlStr += transitionUnknownASPIRE_XML(indent);
		xmlStr += Externaliser.closeInstanceTag(indent);							
		
		return xmlStr;
    }
	
	protected String transitionUnknownASPIRE_XML(String indent) {
		String xmlStr = "";

		// Value
		xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 1, getUnknownValue());
		
		// Unit
		xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 2, getUnknownUnit());
		
		// Transition Label
		xmlStr +=  Externaliser.valueSlotTag(indent+ AppConstants.Sx2, 3, transition.label());
		
		return xmlStr;
	}
	
	public String toString()
	{
		return "Transition " + transition.label() + ": " + type.getName();
	}
}
