package processView;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Point;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.SpringLayout;
import javax.swing.SwingConstants;
import javax.swing.BoxLayout;
import javax.swing.border.LineBorder;

import processModel.SpecialValue;
import processModel.StateElement;
import processModel.Transition;
import processView.ProcessApplet;
import utilities.*;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.ByteArrayOutputStream;
import java.io.PrintWriter;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import processModel.Process;

public class DataSummary extends JPanel{

	private ProcessApplet controller;
	private JScrollPane scrollPane;
	private JPanel workspacePanel;
	
	private java.util.List<Transition> transitions;
	private java.util.List<StateElement> states; 
	
	protected JLabel stateHeader;
	protected JLabel transitionHeader;
	
	private JTextField txtHeat;
	private JTextField txtWork;
	private JTextField txtDelta;
	private JTextField txtTransition;
	private JTextField txtPressure;
	private JTextField txtVolume;
	private JTextField txtTemperature;
	private JTextField txtState;
	private JTextField txtMiscSymbol;
	private JTextField txtMiscValue;
	
	private boolean specialValuesPanelVisible = false;
	private boolean specialValuesComplete = false;
	private SpecialValue specialValueCv;
	private SpecialValue specialValueCp;
	private SpecialValue specialValueR;
	private SpecialValue specialValueGamma;
	private SpecialValue specialValueN;
	
	JPanel transitionsPanel, statesPanel, specialValuesPanel;
	
	public static final int DEFAULT_HEIGHT = 18;
	
	public DataSummary(ProcessApplet controller, java.util.List<Transition> transitions, java.util.List<StateElement> states) {
		this.controller = controller;
		this.states = states;
		this.transitions=transitions;
		
		specialValueCv = null;
		specialValueCp = null;
		specialValueR = null;
		specialValueGamma = null;
		specialValueN = null;
		
		initGUI();
	}
	
	private void initGUI() {
		workspacePanel = new JPanel();
		workspacePanel.setLayout(new BoxLayout(workspacePanel, BoxLayout.X_AXIS));		
		
		scrollPane = new JScrollPane(workspacePanel);		
		scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		scrollPane.setPreferredSize(new Dimension(500,152));
		
		this.setLayout(new BorderLayout());
		//this.add(headPanel, BorderLayout.NORTH);
		this.add(scrollPane, BorderLayout.CENTER);
		
		transitionsPanel = new JPanel(new SpringLayout());
		transitionsPanel.setMaximumSize(new Dimension(350, 10000));
		transitionsPanel.setMinimumSize(new Dimension(350, 350));
		
		specialValuesPanel = new JPanel(new SpringLayout());
		specialValuesPanel.setMaximumSize(new Dimension(350, 10000));
		specialValuesPanel.setMinimumSize(new Dimension(350, 350));
		
		statesPanel = new JPanel(new SpringLayout());
		statesPanel.setMaximumSize(new Dimension(350, 10000));
		statesPanel.setMinimumSize(new Dimension(350, 350));
		
		workspacePanel.add(transitionsPanel);
		workspacePanel.add(specialValuesPanel);
		workspacePanel.add(statesPanel);

		System.out.println("Panel height: " + workspacePanel.getHeight());
	}
	
	public void update()
	{		
		int stateCount = 0;
		int transitionCount = 0;
		transitionsPanel.removeAll();
		statesPanel.removeAll();
		
		JLabel lbl1 = new JLabel(" Transition");
		lbl1.setFont(AppConstants.arialBold);		
		transitionsPanel.add(lbl1);
		JLabel lbl4 = new JLabel(" Work           ");
		lbl4.setFont(AppConstants.arialBold);
		transitionsPanel.add(lbl4);
		JLabel lbl2 = new JLabel(" Heat             ");
		lbl2.setFont(AppConstants.arialBold);
		transitionsPanel.add(lbl2);
		JLabel lbl3 = new JLabel(" Delta U        ");
		lbl3.setFont(AppConstants.arialBold);
		transitionsPanel.add(lbl3);		
		Color darkGreen = new Color(0, 100, 0);
		
		for (Transition element: transitions){
			++transitionCount;
			
			txtTransition = newTextField(false);
			txtTransition.setEditable(false);
			txtTransition.setText(element.label());
			transitionsPanel.add(txtTransition);	
			txtWork = newTextField(true);
			txtWork.setEditable(false);
			txtWork.setText(element.work() + " " + element.getWorkUnits());
			if (element.workCalculated()) {
				txtWork.setForeground(darkGreen);
			}
			transitionsPanel.add(txtWork);
			txtHeat = newTextField(true);
			txtHeat.setEditable(false);
			txtHeat.setText(element.heat() + " " + element.getHeatUnits());
			if (element.heatCalculated()) {
				txtHeat.setForeground(darkGreen);
			}
			transitionsPanel.add(txtHeat);		
			txtDelta = newTextField(true);
			txtDelta.setEditable(false);
			txtDelta.setText(element.delta() + " " + element.getDeltaUnits());
			if (element.deltaCalculated()) {
				txtDelta.setForeground(darkGreen);
			}
			transitionsPanel.add(txtDelta);
		}
		
		SpringUtilities.makeCompactGrid(transitionsPanel, transitionCount +1, 4, 4, 0, 1, 1);
		transitionsPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		
		JLabel lbl5 = new JLabel(" State");
		lbl5.setFont(AppConstants.arialBold);		
		statesPanel.add(lbl5);
		JLabel lbl6 = new JLabel(" Temperature ");
		lbl6.setFont(AppConstants.arialBold);		
		statesPanel.add(lbl6);
		JLabel lbl7 = new JLabel(" Volume        ");
		lbl7.setFont(AppConstants.arialBold);
		statesPanel.add(lbl7);
		JLabel lbl8 = new JLabel(" Pressure    ");
		lbl8.setFont(AppConstants.arialBold);
		statesPanel.add(lbl8);
		
		for (StateElement element: states){
			++stateCount;
			txtState = newTextField(false);
			txtState.setEditable(false);
			txtState.setText(element.label());
			statesPanel.add(txtState);	
			txtTemperature = newTextField(true);
			txtTemperature.setEditable(false);
			txtTemperature.setText(element.temperature() + " " + element.getTemperatureUnits());
			if (element.temperatureCalculated()) {
				txtTemperature.setForeground(darkGreen);
			}
			statesPanel.add(txtTemperature);		
			txtVolume = newTextField(true);
			txtVolume.setEditable(false);
			txtVolume.setText(element.volume() + " " + element.getVolumeUnits());
			if (element.volumeCalculated()) {
				txtVolume.setForeground(darkGreen);
			}
			statesPanel.add(txtVolume);		
			txtPressure = newTextField(true);
			txtPressure.setEditable(false);
			txtPressure.setText(element.pressure() + " " + element.getPressureUnits());
			if (element.pressureCalculated()) {
				txtPressure.setForeground(darkGreen);
			}
			statesPanel.add(txtPressure);
		}
		
		SpringUtilities.makeCompactGrid(statesPanel, stateCount+1, 4, 2, 0, 1, 1);
		statesPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		
		if (specialValuesPanelVisible) 
		{
			specialValuesPanel.removeAll();
			JLabel lb19 = new JLabel(" Symbol");
			lb19.setFont(AppConstants.arialBold);		
			specialValuesPanel.add(lb19);
			JLabel lb20 = new JLabel(" Value");
			lb20.setFont(AppConstants.arialBold);		
			specialValuesPanel.add(lb20);
			JButton btn = null;
			if (!specialValuesComplete) {
				// TODO:  Change back to this
				JLabel lb21 = new JLabel("");
				lb21.setFont(AppConstants.arialBold);
				specialValuesPanel.add(lb21);
				
				/*final JComboBox answerCombo = new JComboBox(new String[] {".", "Dia", "Mon"});
				answerCombo.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent event) {
						if (answerCombo.getSelectedItem().toString().equals("Dia")) {
							try {
								setSpecialValue(new SpecialValue(SpecialValue.SPECIAL_VALUE_CV,
												20.785, Process.getNextID(), new URL("http://www.google.com/"), null));
								setSpecialValue(new SpecialValue(SpecialValue.SPECIAL_VALUE_CP,
												29.099, Process.getNextID(), new URL("http://www.google.com/"), null));
								setSpecialValue(new SpecialValue(SpecialValue.SPECIAL_VALUE_R,
												8.314, Process.getNextID(), new URL("http://www.google.com/"), null));
								setSpecialValue(new SpecialValue(SpecialValue.SPECIAL_VALUE_GAMMA_WORD,
												1.4, Process.getNextID(), new URL("http://www.google.com/"), null));
							}
							catch (Exception ex) {
								ex.printStackTrace();
							}
						}
						else if (answerCombo.getSelectedItem().toString().equals("Mon")) {
							try {
								setSpecialValue(new SpecialValue(SpecialValue.SPECIAL_VALUE_CV,
												12.471, Process.getNextID(), new URL("http://www.google.com/"), null));
								setSpecialValue(new SpecialValue(SpecialValue.SPECIAL_VALUE_CP,
												20.785, Process.getNextID(), new URL("http://www.google.com/"), null));
								setSpecialValue(new SpecialValue(SpecialValue.SPECIAL_VALUE_R,
												8.314, Process.getNextID(), new URL("http://www.google.com/"), null));
								setSpecialValue(new SpecialValue(SpecialValue.SPECIAL_VALUE_GAMMA_WORD,
												1.667, Process.getNextID(), new URL("http://www.google.com/"), null));
							}
							catch (Exception ex) {
								ex.printStackTrace();
							}
						}
					}
				});
				AppUtils.restrictSize(answerCombo, new Dimension(20, 15));
				specialValuesPanel.add(answerCombo);*/
			}
			
			txtMiscSymbol= newTextField(false);
			txtMiscSymbol.setEditable(false);
			txtMiscSymbol.setText(SpecialValue.SPECIAL_VALUE_CV);
			specialValuesPanel.add(txtMiscSymbol);	
			
			txtMiscValue = newTextField(true);
			txtMiscValue.setEditable(false);
			if (specialValueCv != null) {
				txtMiscValue.setText(String.valueOf(specialValueCv.getValue()));
			}
			specialValuesPanel.add(txtMiscValue);
			if (!specialValuesComplete) {
				btn = newButton("...", SpecialValue.SPECIAL_VALUE_CV);
				specialValuesPanel.add(btn);
			}
			
			txtMiscSymbol= newTextField(false);
			txtMiscSymbol.setEditable(false);
			txtMiscSymbol.setText(SpecialValue.SPECIAL_VALUE_CP);
			specialValuesPanel.add(txtMiscSymbol);	
			txtMiscValue = newTextField(true);
			txtMiscValue.setEditable(false);
			if (specialValueCp != null) {
				txtMiscValue.setText(String.valueOf(specialValueCp.getValue()));
			}
			specialValuesPanel.add(txtMiscValue);
			if (!specialValuesComplete) {
				btn = newButton("...", SpecialValue.SPECIAL_VALUE_CP);
				specialValuesPanel.add(btn);
			}
			
			txtMiscSymbol= newTextField(false);
			txtMiscSymbol.setEditable(false);
			txtMiscSymbol.setText(SpecialValue.SPECIAL_VALUE_R);
			specialValuesPanel.add(txtMiscSymbol);	
			txtMiscValue = newTextField(true);
			txtMiscValue.setEditable(false);
			if (specialValueR != null) {
				txtMiscValue.setText(String.valueOf(specialValueR.getValue()));
			}
			specialValuesPanel.add(txtMiscValue);
			if (!specialValuesComplete) {
				btn = newButton("...", SpecialValue.SPECIAL_VALUE_R);
				specialValuesPanel.add(btn);
			}
			
			txtMiscSymbol= newTextField(false);
			txtMiscSymbol.setEditable(false);
			txtMiscSymbol.setText(SpecialValue.SPECIAL_VALUE_GAMMA);
			specialValuesPanel.add(txtMiscSymbol);	
			txtMiscValue = newTextField(true);
			txtMiscValue.setEditable(false);
			if (specialValueGamma != null) {
				txtMiscValue.setText(String.valueOf(specialValueGamma.getValue()));
			}
			specialValuesPanel.add(txtMiscValue);
			if (!specialValuesComplete) {
				btn = newButton("...", SpecialValue.SPECIAL_VALUE_GAMMA);
				specialValuesPanel.add(btn);
			}
			
			txtMiscSymbol= newTextField(false);
			txtMiscSymbol.setEditable(false);
			txtMiscSymbol.setText(SpecialValue.SPECIAL_VALUE_N);
			specialValuesPanel.add(txtMiscSymbol);	
			txtMiscValue = newTextField(true);
			txtMiscValue.setEditable(false);
			if (specialValueN != null) {
				txtMiscValue.setText(String.valueOf(specialValueN.getValue()));
			}
			specialValuesPanel.add(txtMiscValue);
			if (!specialValuesComplete) {
				btn = newButton("...", SpecialValue.SPECIAL_VALUE_N);
				specialValuesPanel.add(btn);
			}
			
			if (!specialValuesComplete) {
				SpringUtilities.makeCompactGrid(specialValuesPanel, 6, 3, 2, 0, 1, 1);
			}
			else {
				SpringUtilities.makeCompactGrid(specialValuesPanel, 6, 2, 2, 0, 1, 1);
			}
			
			if (scrollPane.getViewport().getHeight() < workspacePanel.getHeight()) {
				Point p = scrollPane.getViewport().getViewPosition();
				p.y = workspacePanel.getHeight();
				scrollPane.getViewport().setViewPosition(p);
				System.out.println(scrollPane.getViewport());
			}
		}
		
		this.validate();
	}
	
	public void enableSpecialValuesPanel()
	{
		specialValuesPanelVisible = true;
	}
	
	public void setSpecialValuesComplete()
	{
		specialValuesComplete = true;
	}
	
	public void setSpecialValue(SpecialValue specialValue)
	{
		if (specialValue.getName().equals(SpecialValue.SPECIAL_VALUE_CV)) {
			specialValueCv = specialValue;
		}
		else if (specialValue.getName().equals(SpecialValue.SPECIAL_VALUE_CP)) {
			specialValueCp = specialValue;
		}
		else if (specialValue.getName().equals(SpecialValue.SPECIAL_VALUE_R)) {
			specialValueR = specialValue;
		}
		else if (specialValue.getName().equals(SpecialValue.SPECIAL_VALUE_GAMMA_WORD)) {
			specialValueGamma = specialValue;
		}
		else if (specialValue.getName().equals(SpecialValue.SPECIAL_VALUE_N)) {
			specialValueN = specialValue;
		}
		
		/*if (specialValueCv != null &&
			specialValueCp != null &&
			specialValueR != null &&
			specialValueGamma != null)
		{
			controller.enableFormulaPanel();
		}*/
		
		update();
	}
	
	public String specialValuesToASPIREXML()
	{
		List<SpecialValue> specialValues = new ArrayList<SpecialValue>();
		SpecialValue specialValuesArray[] = new SpecialValue[] {specialValueCv, specialValueCp, specialValueR, specialValueGamma, specialValueN};
		for (SpecialValue value : specialValuesArray) {
			if (value != null) {
				specialValues.add(value);
			}
		}
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		getExternaliser().saveSpecialValuesToASPIREXML(new PrintWriter(bos), controller.getUserName(), controller.getOntologyID(), specialValues);
		return bos.toString();
	}
	
	private Externaliser getExternaliser()
	{
		return controller.getProcess().getExternaliser();
	}
	
	protected JTextField newTextField(boolean addMouseListener) {
		final JTextField txtField = new JTextField(0);
		AppUtils.restrictSize(txtField, new Dimension(60, DEFAULT_HEIGHT));
		final DataSummary ds = this;
		if (addMouseListener) {
			txtField.addMouseListener(new MouseAdapter()
			{
				public void mouseReleased(MouseEvent evt)
				{
					if (evt.getClickCount() == 2 || evt.getButton() == MouseEvent.BUTTON3) {
						ds.controller.getWorkspace().getCurrentFormulaPanel().setSelectedTextField(txtField.getText());
					}
				}
			});
		}
		return txtField;
	}
	
	protected JButton newButton(String label, final String value)
	{
		JButton btn = new JButton(label);
		Dimension btnDim = new Dimension(20, 10);
		btn.setMaximumSize(btnDim);
		btn.setMinimumSize(btnDim);
		btn.setPreferredSize(btnDim);
		final DataSummary ds = this;
		btn.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent evt)
			{
				ds.controller.checkSpecialValueStep(ds, value);
			}
		});
		return btn;
	}
}